from typing import Any, Dict, Union


SELF_CLOSING = (
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
)


class Component:
    def __init__(
        self,
        *slots: Union["Component", str],
        tag_name: str = None,
        attrs: Dict[str, Any] = None,
        inner_html: str = "",
    ):
        self.slots = slots
        self.tag_name = tag_name
        self.attrs = attrs or {}
        self.inner_html = inner_html

    def __repr__(self):
        return f"<Component '{self.__class__.__name__}' attrs={self.attrs}>"

    def proxy(self, other: "Component") -> "Component":
        self.attr(**other.attrs)
        self.inner_html += other.inner_html

        return self

    def attr(self, **attrs) -> "Component":
        self.attrs.update({k: v for k, v in attrs.items() if v})

        return self

    def render(self) -> str:
        if not self.tag_name and self.inner_html:
            return self.inner_html

        if not self.tag_name:
            self.tag_name = "div"

        attrs = ""

        if self.attrs:
            attrs = " " + " ".join(
                ['{}="{}"'.format(k, v) for k, v in self.attrs.items()]
            )

        if self.tag_name in SELF_CLOSING:
            return f"<{self.tag_name}{attrs}>"
        else:
            for slot in self.slots:
                if isinstance(slot, Component):
                    self.inner_html += slot.render()
                else:
                    self.inner_html += slot

            return f"<{self.tag_name}{attrs}>{self.inner_html}</{self.tag_name}>"



from ..component import Component


class Fragment(Component):
    def __init__(self, *slots):
        super().__init__(*slots)

    def render(self):
        return "".join([c.render() for c in self.slots])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['genjuice', 'genjuice.components']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'genjuice',
    'version': '0.1.3',
    'description': 'A lightweight, component-based HTML builder',
    'long_description': '# GenJuice\n\n<p align="center">\n    <img src="https://img.shields.io/pypi/l/genjuice?style=for-the-badge">\n    <img src="https://img.shields.io/pypi/pyversions/genjuice?style=for-the-badge">\n    <img src="https://img.shields.io/github/languages/code-size/docyx/genjuice?style=for-the-badge">\n    <img src="https://img.shields.io/github/issues-raw/docyx/genjuice?style=for-the-badge">\n</p>\n\n\nA lightweight, component-based HTML builder.\n\n```py\nfrom genjuice import Component\nfrom genjuice.components import Button\n\nclass FancyButton(Component):\n    def render(self):\n        return Button("Hello, world!").attr(onclick="alert(1)").render()\n```\n\n```py\n>>> FancyButton().render()\n"""\n<button onclick="alert(1)">Hello, world!</button>\n"""\n```\n\n## Installation\n\nGenJuice isn\'t quite ready for production yet. Feel free to mess around with it though :)\n\n```\npip3 install genjuice\n```\n\n## Why GenJuice?\n\n- The core functionality (`Component.render()`) is 20 lines long\n- On average, it\'s 2.2x faster than its counterparts (HTML builders/template engines such as Jinja)\n- You write your UI code in 100% Python\n- A modern, intuitive API\n- ...and much more to come!\n\n## License\n\n[MIT](./LICENSE)\n',
    'author': 'docyx',
    'author_email': 'oliverxur@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/docyx/genjuice',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
